#!/usr/bin/env python3
# vim:ts=4:sts=4:sw=4:expandtab

import argparse
import logging
import pathlib
import socket
import traceback

DEFAULT_PORT=3456
DEFAULT_DOWNLOADS='./Downloads'

def client_communication(client_socket, server_address, downloads_path, request_path):
    try:
        #TODO: Reliably send request path to server
        logging.info(f'Requested {request_path} from {server_address}')

        downloads_dir = pathlib.Path(downloads_path)
        downloads_dir.mkdir(parents=True, exist_ok=True)
        with (downloads_dir / pathlib.Path(request_path).name).open('wb') as request_file:
            #TODO: Reliably receive request_file from server
            pass

        logging.info(f'Receiving a file {request_path} from {server_address} succeeded')
    except:
        logging.error(f'Receiving a file {request_path} from {server_address} failed:\n{traceback.format_exc()}')

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='File Transfer TCP Client')
    parser.add_argument('--debug', action='store_true')
    parser.add_argument('--host', default='127.0.0.1')
    parser.add_argument('--port', default=DEFAULT_PORT)
    parser.add_argument('--downloads', default=DEFAULT_DOWNLOADS)
    parser.add_argument('path')
    args = parser.parse_args()
    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    with socket.create_connection((args.host, args.port)) as client_socket:
        client_communication(client_socket, client_socket.getpeername(), args.downloads, args.path)
